@@ -1,14 +1,15 @@
-__all__ = ['term', 'logfile', 'github_auth_device', 'limit_cb', 'api', 'fetch_events', 'Events', 'seen', 'print_event', 'tail_events', 'batch_events', 'watch_users', 'quad_logs', 'simple', 'main']
+__all__ = ['term', 'logfile', 'github_auth_device', 'limit_cb', 'api', 'Events', 'print_event', 'tail_events', 'watch_users', 'quad_logs', 'simple', 'main']
 
 import time, sys, signal, shutil, os, json, enlighten, emoji, blessed
 from dashing import *
 from collections import defaultdict
+from warnings import warn
 from itertools import islice
 
 from fastcore.utils import *
 from fastcore.foundation import *
 from fastcore.script import *
-from ghapi.core import *
+from ghapi.all import *
 
 term = Terminal()
 logfile = Path("log.txt")
@@ -47,12 +48,6 @@ def _get_api():
 
 api = _get_api()
 
-def fetch_events(types=None):
-    "Generate an infinite stream of events optionally filtered to `types`"
-    while True:
-        yield from (o for o in api.activity.list_public_events() if not types or o.type in types)
-        time.sleep(0.2)
-
 Events = dict(
     IssuesEvent_closed=('⭐', 'closed', noop),
     IssuesEvent_opened=('📫', 'opened', noop),
@@ -61,8 +56,6 @@ Events = dict(
     PullRequestEvent_closed=('✔', 'closed a pull request', term.green),
 )
 
-seen = set()
-
 def _to_log(e):
     login,repo,pay = e.actor.login,e.repo.name,e.payload
     typ = e.type + (f'_{pay.action}' if e.type in ('PullRequestEvent','IssuesEvent') else '')
@@ -74,11 +67,6 @@ def _to_log(e):
     elif e.type == "ReleaseEvent": return f'🚀 {login} released {e.payload.release.tag_name} of {repo}'
 
 def print_event(e, commits_counter):
-    if e.id in seen: return
-    seen.add(e.id)
-    login = e.actor.login
-    if "bot" in login or "b0t" in login: return  # Don't print bot activity (there is a lot!)
-
     res = _to_log(e)
     if res: print(res)
     elif e.type == "PushEvent": [commits_counter.update() for c in e.payload.commits]
@@ -90,16 +78,12 @@ def tail_events():
     commits = manager.counter(desc='Commits', unit='commits', color='green')
     for ev in fetch_events(): print_event(ev, commits)
 
-def batch_events(size, types=None):
-    "Generate an infinite stream of batches of events of `size` optionally filtered to `types`"
-    while True: yield islice(fetch_events(types=types), size)
-
 def _pr_row(*its): print(f"{its[0]: <30} {its[1]: <6} {its[2]: <5} {its[3]: <6} {its[4]: <7}")
-def watch_users():
+def watch_users(evts):
     "Print a table of the users with the most events"
     users,users_events = defaultdict(int),defaultdict(lambda: defaultdict(int))
-    for xs in batch_events(10):
-        for x in xs:
+    while True:
+        for x in islice(evts, 10):
             users[x.actor.login] += 1
             users_events[x.actor.login][x.type] += 1
 
@@ -112,7 +96,7 @@ def watch_users():
 def _push_to_log(e): return f"{e.actor.login} pushed {len(e.payload.commits)} commits to repo {e.repo.name}"
 def _logwin(title,color): return Log(title=title,border_color=2,color=color)
 
-def quad_logs():
+def quad_logs(evts):
     "Print 4 panels, showing most recent issues, commits, PRs, and releases"
     term.enter_fullscreen()
     ui = HSplit(VSplit(_logwin('Issues',        color=7), _logwin('Commits' , color=3)),
@@ -122,14 +106,14 @@ def quad_logs():
     for o in all_items: o.append(" ")
 
     d = dict(PushEvent=commits, IssuesEvent=issues, IssueCommentEvent=issues, PullRequestEvent=prs, ReleaseEvent=releases)
-    for xs in batch_events(10, types=d):
-        for x in xs:
+    while True:
+        for x in islice(evts, 10):
             f = [_to_log,_push_to_log][x.type == 'PushEvent']
-            d[x.type].append(f(x)[:95])
+            if x.type in d: d[x.type].append(f(x)[:95])
         ui.display()
 
-def simple():
-    for ev in fetch_events(): print(f"{ev.actor.login} {ev.type} {ev.repo.name}")
+def simple(evts):
+    for ev in evts: print(f"{ev.actor.login} {ev.type} {ev.repo.name}")
 
 def _signal_handler(sig, frame):
     if sig != signal.SIGINT: return
@@ -137,10 +121,19 @@ def _signal_handler(sig, frame):
     sys.exit(0)
 
 _funcs = dict(tail=tail_events, quad=quad_logs, users=watch_users, simple=simple)
+_filts = str_enum('_filts', 'user', 'repo', 'org')
 _OpModes = str_enum('_OpModes', *_funcs)
 
 @call_parse
-def main(mode: Param("Operation mode to run", _OpModes)):
+def main(mode:         Param("Operation mode to run", _OpModes),
+         include_bots: Param("Include bots (there's a lot of them!)", store_true)=False,
+         types:        Param("Comma-separated types of event to include (e.g PushEvent)", str)='',
+         filt:         Param("Filtering method", _filts)=None,
+         filtval:      Param("Value to filter by (for `repo` use format `owner/repo`)", str)=None):
     signal.signal(signal.SIGINT, _signal_handler)
-    _funcs[mode]()
-
+    types = types.split(',') if types else None
+    if filt and not filtval: _exit("Must pass `filter_value` if passing `filter_type`")
+    if filtval and not filt: _exit("Must pass `filter_type` if passing `filter_value`")
+    kwargs = {filt:filtval} if filt else {}
+    evts = api.fetch_events(types=types, incl_bot=include_bots, **kwargs)
+    _funcs[mode](evts)
